@@ -91,6 +91,7 @@
 #define INPUT_SOURCE_TYPE_IBUS "ibus"
 
 #define DEFAULT_LANGUAGE "en_US"
+#define DEFAULT_LAYOUT "us"
 
 struct GsdKeyboardManagerPrivate
 {
@@ -100,6 +101,7 @@ struct GsdKeyboardManagerPrivate
         GSettings *interface_settings;
         GnomeXkbInfo *xkb_info;
         GDBusProxy *localed;
+        GCancellable *cancellable;
 #ifdef HAVE_IBUS
         IBusBus   *ibus;
         GHashTable *ibus_engines;
@@ -668,6 +670,11 @@ upload_xkb_description (const gchar          *rules_file_path,
         XkbDescRec *xkb_desc;
         gchar *rules_file;
 
+        /* The layout we want is always in the first XKB group index
+         * so we should enforce it to make sure we never end up with
+         * the wrong one. */
+        XkbLockGroup (display, XkbUseCoreKbd, 0);
+
         /* Upload it to the X server using the same method as setxkbmap */
         xkb_desc = XkbGetKeyboardByName (display,
                                          XkbUseCoreKbd,
@@ -1264,6 +1271,7 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
         gint i, n;
         gchar **layouts = NULL;
         gchar **variants = NULL;
+        gboolean have_default_layout = FALSE;
 
         v = g_dbus_proxy_get_cached_property (priv->localed, "X11Layout");
         if (v) {
@@ -1299,9 +1307,16 @@ get_sources_from_xkb_config (GsdKeyboardManager *manager)
                 else
                         id = g_strdup (layouts[i]);
 
+                if (g_str_equal (id, DEFAULT_LAYOUT))
+                        have_default_layout = TRUE;
+
                 g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, id);
                 g_free (id);
         }
+
+        if (!have_default_layout)
+                g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, DEFAULT_LAYOUT);
+
         g_settings_set_value (priv->input_sources_settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
 
         g_strfreev (layouts);
@@ -1495,16 +1510,21 @@ localed_proxy_ready (GObject      *source,
                      gpointer      data)
 {
         GsdKeyboardManager *manager = data;
+        GDBusProxy *proxy;
         GError *error = NULL;
 
-        manager->priv->localed = g_dbus_proxy_new_finish (res, &error);
-
-        if (!manager->priv->localed) {
+        proxy = g_dbus_proxy_new_finish (res, &error);
+        if (!proxy) {
+                if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        g_error_free (error);
+                        return;
+                }
                 g_warning ("Failed to contact localed: %s", error->message);
                 g_error_free (error);
                 goto out;
         }
 
+        manager->priv->localed = proxy;
         maybe_create_initial_settings (manager);
 out:
         apply_input_sources_settings (manager->priv->input_sources_settings, NULL, 0, manager);
@@ -1527,13 +1547,15 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         manager->priv->interface_settings = g_settings_new (GNOME_DESKTOP_INTERFACE_DIR);
         manager->priv->xkb_info = gnome_xkb_info_new ();
 
+        manager->priv->cancellable = g_cancellable_new ();
+
         g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
                                   G_DBUS_PROXY_FLAGS_NONE,
                                   NULL,
                                   "org.freedesktop.locale1",
                                   "/org/freedesktop/locale1",
                                   "org.freedesktop.locale1",
-                                  NULL,
+                                  manager->priv->cancellable,
                                   localed_proxy_ready,
                                   manager);
 
@@ -1581,6 +1603,9 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
 
         g_debug ("Stopping keyboard manager");
 
+        g_cancellable_cancel (p->cancellable);
+        g_clear_object (&p->cancellable);
+
         g_clear_object (&p->settings);
         g_clear_object (&p->input_sources_settings);
         g_clear_object (&p->interface_settings);
