@@ -266,6 +266,7 @@ get_keyring_env (GsdMediaKeysManager *manager)
 	}
 
 	envp = g_get_environ ();
+	envp = g_environ_unsetenv (envp, "DESKTOP_AUTOSTART_ID");
 
 	g_variant_get (variant, "(a{ss})", &iter);
 
@@ -370,15 +371,6 @@ ensure_cancellable (GCancellable **cancellable)
         }
 }
 
-static void
-show_osd_complete (GObject      *source,
-                   GAsyncResult *result,
-                   gpointer      data)
-{
-        GsdMediaKeysManager *manager = data;
-        g_object_unref (manager->priv->osd_cancellable);
-}
-
 static void
 show_osd (GsdMediaKeysManager *manager,
           const char          *icon,
@@ -403,14 +395,13 @@ show_osd (GsdMediaKeysManager *manager,
                                        "level", g_variant_new_int32 (level));
         g_variant_builder_close (&builder);
 
-        ensure_cancellable (&manager->priv->osd_cancellable);
         g_dbus_proxy_call (manager->priv->osd_proxy,
                            "ShowOSD",
                            g_variant_builder_end (&builder),
                            G_DBUS_CALL_FLAGS_NO_AUTO_START,
                            -1,
                            manager->priv->osd_cancellable,
-                           show_osd_complete, manager);
+                           NULL, NULL);
 }
 
 static const char *
@@ -472,9 +463,7 @@ grab_accelerators_complete (GObject      *object,
         GsdMediaKeysManager *manager = user_data;
 
         shell_key_grabber_call_grab_accelerators_finish (SHELL_KEY_GRABBER (object),
-                                                         &actions, result, &error); 
-
-        g_object_unref (manager->priv->grab_cancellable);
+                                                         &actions, result, &error);
 
         if (error) {
                 retry = (error->code == G_DBUS_ERROR_UNKNOWN_METHOD);
@@ -514,7 +503,6 @@ grab_media_keys (GsdMediaKeysManager *manager)
                 g_free (tmp);
         }
 
-	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_grab_accelerators (manager->priv->key_grabber,
 	                                          g_variant_builder_end (&builder),
 	                                          manager->priv->grab_cancellable,
@@ -531,9 +519,8 @@ grab_accelerator_complete (GObject      *object,
         MediaKey *key = data->key;
 
         shell_key_grabber_call_grab_accelerator_finish (SHELL_KEY_GRABBER (object),
-                                                        &key->accel_id, result, NULL); 
+                                                        &key->accel_id, result, NULL);
 
-        g_object_unref (data->manager->priv->grab_cancellable);
         g_slice_free (GrabData, data);
 }
 
@@ -552,7 +539,6 @@ grab_media_key (MediaKey            *key,
 	data->manager = manager;
 	data->key = key;
 
-	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_grab_accelerator (manager->priv->key_grabber,
 	                                         tmp, key->modes,
 	                                         manager->priv->grab_cancellable,
@@ -570,7 +556,6 @@ ungrab_accelerator_complete (GObject      *object,
 	GsdMediaKeysManager *manager = user_data;
 	shell_key_grabber_call_ungrab_accelerator_finish (SHELL_KEY_GRABBER (object),
 	                                                  NULL, result, NULL);
-	g_object_unref (manager->priv->grab_cancellable);
 }
 
 static void
@@ -580,7 +565,6 @@ ungrab_media_key (MediaKey            *key,
 	if (key->accel_id == 0)
 		return;
 
-	ensure_cancellable (&manager->priv->grab_cancellable);
 	shell_key_grabber_call_ungrab_accelerator (manager->priv->key_grabber,
 	                                           key->accel_id,
 	                                           manager->priv->grab_cancellable,
@@ -596,6 +580,10 @@ gsettings_changed_cb (GSettings           *settings,
 {
         int      i;
 
+        /* Give up if we don't have proxy to the shell */
+        if (!manager->priv->key_grabber)
+                return;
+
 	/* handled in gsettings_custom_changed_cb() */
         if (g_str_equal (settings_key, "custom-keybindings"))
 		return;
@@ -765,8 +753,6 @@ init_kbd (GsdMediaKeysManager *manager)
 
         gnome_settings_profile_start (NULL);
 
-        manager->priv->keys = g_ptr_array_new_with_free_func ((GDestroyNotify) media_key_free);
-
         /* Media keys
          * Add hard-coded shortcuts first so that they can't be preempted */
         for (i = 0; i < G_N_ELEMENTS (media_keys); i++) {
@@ -1050,7 +1036,7 @@ update_dialog (GsdMediaKeysManager *manager,
         icon = get_icon_name_for_volume (!GVC_IS_MIXER_SINK (stream), muted, vol);
         port = gvc_mixer_stream_get_port (stream);
         if (g_strcmp0 (gvc_mixer_stream_get_form_factor (stream), "internal") != 0 ||
-            g_strcmp0 (port->port, "analog-output-speaker") != 0) {
+            (port != NULL && g_strcmp0 (port->port, "analog-output-speaker") != 0)) {
                 device = gvc_mixer_control_lookup_device_from_stream (manager->priv->volume, stream);
                 show_osd (manager, icon,
                           gvc_mixer_ui_device_get_description (device), vol);
@@ -2195,7 +2181,6 @@ on_osd_proxy_ready (GObject      *source,
 
         manager->priv->osd_proxy =
                 g_dbus_proxy_new_for_bus_finish (result, NULL);
-        g_object_unref (manager->priv->osd_cancellable);
 }
 
 static void
@@ -2208,8 +2193,6 @@ on_key_grabber_ready (GObject      *source,
         manager->priv->key_grabber =
 		shell_key_grabber_proxy_new_for_bus_finish (result, NULL);
 
-        g_object_unref (manager->priv->grab_cancellable);
-
         if (!manager->priv->key_grabber)
                 return;
 
@@ -2226,15 +2209,14 @@ on_shell_appeared (GDBusConnection   *connection,
                    gpointer           user_data)
 {
         GsdMediaKeysManager *manager = user_data;
-        
+
         shell_key_grabber_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                                              0,
                                              name_owner,
                                              SHELL_DBUS_PATH,
                                              manager->priv->grab_cancellable,
                                              on_key_grabber_ready, manager);
-        
-        ensure_cancellable (&manager->priv->osd_cancellable);
+
         g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                                   0, NULL,
                                   name_owner,
@@ -2250,6 +2232,9 @@ on_shell_vanished (GDBusConnection  *connection,
                    gpointer          user_data)
 {
         GsdMediaKeysManager *manager = user_data;
+
+        g_ptr_array_set_size (manager->priv->keys, 0);
+
         g_clear_object (&manager->priv->key_grabber);
         g_clear_object (&manager->priv->osd_proxy);
 }
@@ -2262,6 +2247,8 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
         g_debug ("Starting media_keys manager");
         gnome_settings_profile_start (NULL);
 
+        manager->priv->keys = g_ptr_array_new_with_free_func ((GDestroyNotify) media_key_free);
+
         initialize_volume_handler (manager);
 
         manager->priv->settings = g_settings_new (SETTINGS_BINDING_DIR);
@@ -2305,6 +2292,7 @@ start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 	manager->priv->icon_theme = g_settings_get_string (manager->priv->interface_settings, "icon-theme");
 
         ensure_cancellable (&manager->priv->grab_cancellable);
+        ensure_cancellable (&manager->priv->osd_cancellable);
 
         manager->priv->name_owner_id =
                 g_bus_watch_name (G_BUS_TYPE_SESSION,
