@@ -21,11 +21,37 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <sys/wait.h>
 #include <math.h>
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gdk/gdkx.h>
+#include <X11/extensions/XTest.h>
+#include <canberra-gtk.h>
+
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnome-desktop/gnome-rr.h>
 
 #include "gpm-common.h"
+#include "gsd-power-constants.h"
+#include "gsd-power-manager.h"
+#include "gsd-backlight-linux.h"
+
+#define XSCREENSAVER_WATCHDOG_TIMEOUT           120 /* seconds */
+#define UPS_SOUND_LOOP_ID                        99
+#define GSD_POWER_MANAGER_CRITICAL_ALERT_TIMEOUT  5 /* seconds */
+
+/* take a discrete value with offset and convert to percentage */
+int
+gsd_power_backlight_abs_to_percentage (int min, int max, int value)
+{
+        g_return_val_if_fail (max > min, -1);
+        g_return_val_if_fail (value >= min, -1);
+        g_return_val_if_fail (value <= max, -1);
+        return (((value - min) * 100) / (max - min));
+}
 
 #define GPM_UP_TIME_PRECISION                   5*60
 #define GPM_UP_TEXT_MIN_TIME                    120
@@ -751,12 +777,6 @@ gpm_device_to_localised_string (UpDevice *device)
                       "state", &state,
                       NULL);
 
-        /* line power */
-        if (kind == UP_DEVICE_KIND_LINE_POWER) {
-                /* TRANSLATORS: system power cord */
-                return _("AC adapter");
-        }
-
         /* laptop battery */
         if (kind == UP_DEVICE_KIND_BATTERY) {
 
@@ -788,9 +808,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Laptop battery is waiting to discharge");
                 }
-
-                /* TRANSLATORS: laptop primary battery */
-                return _("Laptop battery");
         }
 
         /* UPS */
@@ -812,16 +829,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("UPS is charged");
                 }
-
-                /* TRANSLATORS: battery-backed AC power source */
-                return _("UPS");
-        }
-
-        /* monitor */
-        if (kind == UP_DEVICE_KIND_MONITOR) {
-
-                /* TRANSLATORS: a monitor is a device to measure voltage and current */
-                return _("Monitor");
         }
 
         /* mouse */
@@ -843,9 +850,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Mouse is charged");
                 }
-
-                /* TRANSLATORS: wireless mice with internal batteries */
-                return _("Mouse");
         }
 
         /* keyboard */
@@ -867,9 +871,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Keyboard is charged");
                 }
-
-                /* TRANSLATORS: wireless keyboard with internal battery */
-                return _("Keyboard");
         }
 
         /* PDA */
@@ -891,9 +892,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("PDA is charged");
                 }
-
-                /* TRANSLATORS: portable device */
-                return _("PDA");
         }
 
         /* phone */
@@ -915,9 +913,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Cell phone is charged");
                 }
-
-                /* TRANSLATORS: cell phone (mobile...) */
-                return _("Cell phone");
         }
 #if UP_CHECK_VERSION(0,9,5)
 
@@ -940,9 +935,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Media player is charged");
                 }
-
-                /* TRANSLATORS: media player, mp3 etc */
-                return _("Media player");
         }
 
         /* tablet */
@@ -964,9 +956,6 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Tablet is charged");
                 }
-
-                /* TRANSLATORS: tablet device */
-                return _("Tablet");
         }
 
         /* computer */
@@ -988,10 +977,787 @@ gpm_device_to_localised_string (UpDevice *device)
                         /* TRANSLATORS: battery state */
                         return _("Computer is charged");
                 }
-
-                /* TRANSLATORS: tablet device */
-                return _("Computer");
         }
 #endif
-        return NULL;
+
+        return gpm_device_kind_to_localised_string (kind, 1);
+}
+
+static gboolean
+parse_vm_kernel_cmdline (gboolean *is_virtual_machine)
+{
+        gboolean ret = FALSE;
+        GRegex *regex;
+        GMatchInfo *match;
+        char *contents;
+        char *word;
+        const char *arg;
+
+        if (!g_file_get_contents ("/proc/cmdline", &contents, NULL, NULL))
+                return ret;
+
+        regex = g_regex_new ("gnome.is_vm=(\\S+)", 0, G_REGEX_MATCH_NOTEMPTY, NULL);
+        if (!g_regex_match (regex, contents, G_REGEX_MATCH_NOTEMPTY, &match))
+                goto out;
+
+        word = g_match_info_fetch (match, 0);
+        g_debug ("Found command-line match '%s'", word);
+        arg = word + strlen ("gnome.is_vm=");
+        if (*arg != '0' && *arg != '1') {
+                g_warning ("Invalid value '%s' for gnome.is_vm passed in kernel command line.\n", arg);
+        } else {
+                *is_virtual_machine = atoi (arg);
+                ret = TRUE;
+        }
+        g_free (word);
+
+out:
+        g_match_info_free (match);
+        g_regex_unref (regex);
+        g_free (contents);
+
+        if (ret)
+                g_debug ("Kernel command-line parsed to %d", *is_virtual_machine);
+
+        return ret;
+}
+
+gboolean
+gsd_power_is_hardware_a_vm (void)
+{
+        const gchar *str;
+        gboolean ret = FALSE;
+        GError *error = NULL;
+        GVariant *inner;
+        GVariant *variant = NULL;
+        GDBusConnection *connection;
+
+        if (parse_vm_kernel_cmdline (&ret))
+                return ret;
+
+        connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM,
+                                     NULL,
+                                     &error);
+        if (connection == NULL) {
+                g_warning ("system bus not available: %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+        variant = g_dbus_connection_call_sync (connection,
+                                               "org.freedesktop.systemd1",
+                                               "/org/freedesktop/systemd1",
+                                               "org.freedesktop.DBus.Properties",
+                                               "Get",
+                                               g_variant_new ("(ss)",
+                                                              "org.freedesktop.systemd1.Manager",
+                                                              "Virtualization"),
+                                               NULL,
+                                               G_DBUS_CALL_FLAGS_NONE,
+                                               -1,
+                                               NULL,
+                                               &error);
+        if (variant == NULL) {
+                g_debug ("Failed to get property '%s': %s", "Virtualization", error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        /* on bare-metal hardware this is the empty string,
+         * otherwise an identifier such as "kvm", "vmware", etc. */
+        g_variant_get (variant, "(v)", &inner);
+        str = g_variant_get_string (inner, NULL);
+        if (str != NULL && str[0] != '\0')
+                ret = TRUE;
+out:
+        if (connection != NULL)
+                g_object_unref (connection);
+        if (variant != NULL)
+                g_variant_unref (variant);
+        return ret;
+}
+
+/* This timer goes off every few minutes, whether the user is idle or not,
+   to try and clean up anything that has gone wrong.
+
+   It calls disable_builtin_screensaver() so that if xset has been used,
+   or some other program (like xlock) has messed with the XSetScreenSaver()
+   settings, they will be set back to sensible values (if a server extension
+   is in use, messing with xlock can cause the screensaver to never get a wakeup
+   event, and could cause monitor power-saving to occur, and all manner of
+   heinousness.)
+
+   This code was originally part of gnome-screensaver, see
+   http://git.gnome.org/browse/gnome-screensaver/tree/src/gs-watcher-x11.c?id=fec00b12ec46c86334cfd36b37771cc4632f0d4d#n530
+ */
+static gboolean
+disable_builtin_screensaver (gpointer unused)
+{
+        int current_server_timeout, current_server_interval;
+        int current_prefer_blank,   current_allow_exp;
+        int desired_server_timeout, desired_server_interval;
+        int desired_prefer_blank,   desired_allow_exp;
+
+        XGetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                         &current_server_timeout,
+                         &current_server_interval,
+                         &current_prefer_blank,
+                         &current_allow_exp);
+
+        desired_server_timeout  = current_server_timeout;
+        desired_server_interval = current_server_interval;
+        desired_prefer_blank    = current_prefer_blank;
+        desired_allow_exp       = current_allow_exp;
+
+        desired_server_interval = 0;
+
+        /* I suspect (but am not sure) that DontAllowExposures might have
+           something to do with powering off the monitor as well, at least
+           on some systems that don't support XDPMS?  Who know... */
+        desired_allow_exp = AllowExposures;
+
+        /* When we're not using an extension, set the server-side timeout to 0,
+           so that the server never gets involved with screen blanking, and we
+           do it all ourselves.  (However, when we *are* using an extension,
+           we tell the server when to notify us, and rather than blanking the
+           screen, the server will send us an X event telling us to blank.)
+        */
+        desired_server_timeout = 0;
+
+        if (desired_server_timeout     != current_server_timeout
+            || desired_server_interval != current_server_interval
+            || desired_prefer_blank    != current_prefer_blank
+            || desired_allow_exp       != current_allow_exp) {
+
+                g_debug ("disabling server builtin screensaver:"
+                         " (xset s %d %d; xset s %s; xset s %s)",
+                         desired_server_timeout,
+                         desired_server_interval,
+                         (desired_prefer_blank ? "blank" : "noblank"),
+                         (desired_allow_exp ? "expose" : "noexpose"));
+
+                XSetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                                 desired_server_timeout,
+                                 desired_server_interval,
+                                 desired_prefer_blank,
+                                 desired_allow_exp);
+
+                XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
+        }
+
+        return TRUE;
+}
+
+guint
+gsd_power_enable_screensaver_watchdog (void)
+{
+        return g_timeout_add_seconds (XSCREENSAVER_WATCHDOG_TIMEOUT,
+                                      disable_builtin_screensaver,
+                                      NULL);
+}
+
+static GnomeRROutput *
+get_primary_output (GnomeRRScreen *rr_screen)
+{
+        GnomeRROutput *output = NULL;
+        GnomeRROutput **outputs;
+        guint i;
+
+        /* search all X11 outputs for the device id */
+        outputs = gnome_rr_screen_list_outputs (rr_screen);
+        if (outputs == NULL)
+                goto out;
+
+        for (i = 0; outputs[i] != NULL; i++) {
+                if (gnome_rr_output_is_connected (outputs[i]) &&
+                    gnome_rr_output_is_laptop (outputs[i]) &&
+                    gnome_rr_output_get_backlight_min (outputs[i]) >= 0 &&
+                    gnome_rr_output_get_backlight_max (outputs[i]) > 0) {
+                        output = outputs[i];
+                        break;
+                }
+        }
+out:
+        return output;
+}
+
+#ifdef GSD_MOCK
+static void
+backlight_set_mock_value (gint value)
+{
+	const char *filename;
+	char *contents;
+
+	g_debug ("Settings mock brightness: %d", value);
+
+	filename = "GSD_MOCK_brightness";
+	contents = g_strdup_printf ("%d", value);
+	g_file_set_contents (filename, contents, -1, NULL);
+	g_free (contents);
+}
+
+static gint64
+backlight_get_mock_value (const char *argument)
+{
+	const char *filename;
+	char *contents;
+	gint64 ret;
+
+	if (g_str_equal (argument, "get-max-brightness")) {
+		g_debug ("Returning max mock brightness: %d", GSD_MOCK_MAX_BRIGHTNESS);
+		return GSD_MOCK_MAX_BRIGHTNESS;
+	}
+
+	if (g_str_equal (argument, "get-brightness")) {
+		filename = "GSD_MOCK_brightness";
+		ret = GSD_MOCK_DEFAULT_BRIGHTNESS;
+	} else {
+		g_assert_not_reached ();
+	}
+
+	if (g_file_get_contents (filename, &contents, NULL, NULL)) {
+		ret = g_ascii_strtoll (contents, NULL, 0);
+		g_free (contents);
+		g_debug ("Returning mock brightness: %"G_GINT64_FORMAT, ret);
+	} else {
+		ret = GSD_MOCK_DEFAULT_BRIGHTNESS;
+		backlight_set_mock_value (GSD_MOCK_DEFAULT_BRIGHTNESS);
+		g_debug ("Returning default mock brightness: %"G_GINT64_FORMAT, ret);
+	}
+
+	return ret;
+}
+#endif /* GSD_MOCK */
+
+gboolean
+backlight_available (GnomeRRScreen *rr_screen)
+{
+        char *path;
+
+#ifdef GSD_MOCK
+	return TRUE;
+#endif
+        if (get_primary_output (rr_screen) != NULL)
+                return TRUE;
+        path = gsd_backlight_helper_get_best_backlight ();
+        if (path == NULL)
+                return FALSE;
+
+        g_free (path);
+        return TRUE;
+}
+
+/**
+ * backlight_helper_get_value:
+ *
+ * Gets a brightness value from the PolicyKit helper.
+ *
+ * Return value: the signed integer value from the helper, or -1
+ * for failure. If -1 then @error is set.
+ **/
+static gint64
+backlight_helper_get_value (const gchar *argument, GError **error)
+{
+        gboolean ret;
+        gchar *stdout_data = NULL;
+        gint exit_status = 0;
+        gint64 value = -1;
+        gchar *command = NULL;
+        gchar *endptr = NULL;
+
+#ifdef GSD_MOCK
+        return backlight_get_mock_value (argument);
+#endif
+
+#ifndef __linux__
+        /* non-Linux platforms won't have /sys/class/backlight */
+        g_set_error_literal (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "The sysfs backlight helper is only for Linux");
+        goto out;
+#endif
+
+        /* get the data */
+        command = g_strdup_printf (LIBEXECDIR "/gsd-backlight-helper --%s",
+                                   argument);
+        ret = g_spawn_command_line_sync (command,
+                                         &stdout_data,
+                                         NULL,
+                                         &exit_status,
+                                         error);
+        g_debug ("executed %s retval: %i", command, exit_status);
+
+        if (!ret)
+                goto out;
+
+        if (WEXITSTATUS (exit_status) != 0) {
+                 g_set_error (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "gsd-backlight-helper failed: %s",
+                             stdout_data ? stdout_data : "No reason");
+                goto out;
+        }
+
+        /* parse */
+        value = g_ascii_strtoll (stdout_data, &endptr, 10);
+
+        /* parsing error */
+        if (endptr == stdout_data) {
+                value = -1;
+                g_set_error (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "failed to parse value: %s",
+                             stdout_data);
+                goto out;
+        }
+
+        /* out of range */
+        if (value > G_MAXINT) {
+                value = -1;
+                g_set_error (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "value out of range: %s",
+                             stdout_data);
+                goto out;
+        }
+
+        /* Fetching the value failed, for some other reason */
+        if (value < 0) {
+                g_set_error (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "value negative, but helper did not fail: %s",
+                             stdout_data);
+                goto out;
+        }
+
+out:
+        g_free (command);
+        g_free (stdout_data);
+        return value;
+}
+
+/**
+ * backlight_helper_set_value:
+ *
+ * Sets a brightness value using the PolicyKit helper.
+ *
+ * Return value: Success. If FALSE then @error is set.
+ **/
+static gboolean
+backlight_helper_set_value (const gchar *argument,
+                            gint value,
+                            GError **error)
+{
+        gboolean ret = FALSE;
+        gint exit_status = 0;
+        gchar *command = NULL;
+
+#ifdef GSD_MOCK
+	backlight_set_mock_value (value);
+	return TRUE;
+#endif
+
+#ifndef __linux__
+        /* non-Linux platforms won't have /sys/class/backlight */
+        g_set_error_literal (error,
+                             GSD_POWER_MANAGER_ERROR,
+                             GSD_POWER_MANAGER_ERROR_FAILED,
+                             "The sysfs backlight helper is only for Linux");
+        goto out;
+#endif
+
+        /* get the data */
+        command = g_strdup_printf ("pkexec " LIBEXECDIR "/gsd-backlight-helper --%s %i",
+                                   argument, value);
+        ret = g_spawn_command_line_sync (command,
+                                         NULL,
+                                         NULL,
+                                         &exit_status,
+                                         error);
+
+        g_debug ("executed %s retval: %i", command, exit_status);
+
+        if (!ret || WEXITSTATUS (exit_status) != 0)
+                goto out;
+
+out:
+        g_free (command);
+        return ret;
+}
+
+int
+backlight_get_abs (GnomeRRScreen *rr_screen, GError **error)
+{
+        GnomeRROutput *output;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+                return gnome_rr_output_get_backlight (output,
+                                                      error);
+        }
+
+        /* fall back to the polkit helper */
+        return backlight_helper_get_value ("get-brightness", error);
+}
+
+int
+backlight_get_percentage (GnomeRRScreen *rr_screen, GError **error)
+{
+        GnomeRROutput *output;
+        gint now;
+        gint value = -1;
+        gint min = 0;
+        gint max;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+
+                min = gnome_rr_output_get_backlight_min (output);
+                max = gnome_rr_output_get_backlight_max (output);
+                now = gnome_rr_output_get_backlight (output, error);
+                if (now < 0)
+                        goto out;
+                value = ABS_TO_PERCENTAGE (min, max, now);
+                goto out;
+        }
+
+        /* fall back to the polkit helper */
+        max = backlight_helper_get_value ("get-max-brightness", error);
+        if (max < 0)
+                goto out;
+        now = backlight_helper_get_value ("get-brightness", error);
+        if (now < 0)
+                goto out;
+        value = ABS_TO_PERCENTAGE (min, max, now);
+out:
+        return value;
+}
+
+int
+backlight_get_min (GnomeRRScreen *rr_screen)
+{
+        GnomeRROutput *output;
+
+        /* if we have no xbacklight device, then hardcode zero as sysfs
+         * offsets everything to 0 as min */
+        output = get_primary_output (rr_screen);
+        if (output == NULL)
+                return 0;
+
+        /* get xbacklight value, which maybe non-zero */
+        return gnome_rr_output_get_backlight_min (output);
+}
+
+int
+backlight_get_max (GnomeRRScreen *rr_screen, GError **error)
+{
+        gint value;
+        GnomeRROutput *output;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+                value = gnome_rr_output_get_backlight_max (output);
+                if (value < 0) {
+                        g_set_error (error,
+                                     GSD_POWER_MANAGER_ERROR,
+                                     GSD_POWER_MANAGER_ERROR_FAILED,
+                                     "failed to get backlight max");
+                }
+                return value;
+        }
+
+        /* fall back to the polkit helper */
+        return  backlight_helper_get_value ("get-max-brightness", error);
+}
+
+gboolean
+backlight_set_percentage (GnomeRRScreen *rr_screen,
+                          guint value,
+                          GError **error)
+{
+        GnomeRROutput *output;
+        gboolean ret = FALSE;
+        gint min = 0;
+        gint max;
+        guint discrete;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+                min = gnome_rr_output_get_backlight_min (output);
+                max = gnome_rr_output_get_backlight_max (output);
+                if (min < 0 || max < 0) {
+                        g_warning ("no xrandr backlight capability");
+                        return ret;
+                }
+                discrete = PERCENTAGE_TO_ABS (min, max, value);
+                ret = gnome_rr_output_set_backlight (output,
+                                                     discrete,
+                                                     error);
+                return ret;
+        }
+
+        /* fall back to the polkit helper */
+        max = backlight_helper_get_value ("get-max-brightness", error);
+        if (max < 0)
+                return ret;
+        discrete = PERCENTAGE_TO_ABS (min, max, value);
+        ret = backlight_helper_set_value ("set-brightness",
+                                          discrete,
+                                          error);
+
+        return ret;
+}
+
+int
+backlight_step_up (GnomeRRScreen *rr_screen, GError **error)
+{
+        GnomeRROutput *output;
+        gboolean ret = FALSE;
+        gint percentage_value = -1;
+        gint min = 0;
+        gint max;
+        gint now;
+        gint step;
+        guint discrete;
+        GnomeRRCrtc *crtc;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+
+                crtc = gnome_rr_output_get_crtc (output);
+                if (crtc == NULL) {
+                        g_set_error (error,
+                                     GSD_POWER_MANAGER_ERROR,
+                                     GSD_POWER_MANAGER_ERROR_FAILED,
+                                     "no crtc for %s",
+                                     gnome_rr_output_get_name (output));
+                        return percentage_value;
+                }
+                min = gnome_rr_output_get_backlight_min (output);
+                max = gnome_rr_output_get_backlight_max (output);
+                now = gnome_rr_output_get_backlight (output, error);
+                if (now < 0)
+                       return percentage_value;
+                step = BRIGHTNESS_STEP_AMOUNT (max - min + 1);
+                discrete = MIN (now + step, max);
+                ret = gnome_rr_output_set_backlight (output,
+                                                     discrete,
+                                                     error);
+                if (ret)
+                        percentage_value = ABS_TO_PERCENTAGE (min, max, discrete);
+                return percentage_value;
+        }
+
+        /* fall back to the polkit helper */
+        now = backlight_helper_get_value ("get-brightness", error);
+        if (now < 0)
+                return percentage_value;
+        max = backlight_helper_get_value ("get-max-brightness", error);
+        if (max < 0)
+                return percentage_value;
+        step = BRIGHTNESS_STEP_AMOUNT (max - min + 1);
+        discrete = MIN (now + step, max);
+        ret = backlight_helper_set_value ("set-brightness",
+                                          discrete,
+                                          error);
+        if (ret)
+                percentage_value = ABS_TO_PERCENTAGE (min, max, discrete);
+
+        return percentage_value;
+}
+
+int
+backlight_step_down (GnomeRRScreen *rr_screen, GError **error)
+{
+        GnomeRROutput *output;
+        gboolean ret = FALSE;
+        gint percentage_value = -1;
+        gint min = 0;
+        gint max;
+        gint now;
+        gint step;
+        guint discrete;
+        GnomeRRCrtc *crtc;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+
+                crtc = gnome_rr_output_get_crtc (output);
+                if (crtc == NULL) {
+                        g_set_error (error,
+                                     GSD_POWER_MANAGER_ERROR,
+                                     GSD_POWER_MANAGER_ERROR_FAILED,
+                                     "no crtc for %s",
+                                     gnome_rr_output_get_name (output));
+                        return percentage_value;
+                }
+                min = gnome_rr_output_get_backlight_min (output);
+                max = gnome_rr_output_get_backlight_max (output);
+                now = gnome_rr_output_get_backlight (output, error);
+                if (now < 0)
+                       return percentage_value;
+                step = BRIGHTNESS_STEP_AMOUNT (max - min + 1);
+                discrete = MAX (now - step, 0);
+                ret = gnome_rr_output_set_backlight (output,
+                                                     discrete,
+                                                     error);
+                if (ret)
+                        percentage_value = ABS_TO_PERCENTAGE (min, max, discrete);
+                return percentage_value;
+        }
+
+        /* fall back to the polkit helper */
+        now = backlight_helper_get_value ("get-brightness", error);
+        if (now < 0)
+                return percentage_value;
+        max = backlight_helper_get_value ("get-max-brightness", error);
+        if (max < 0)
+                return percentage_value;
+        step = BRIGHTNESS_STEP_AMOUNT (max - min + 1);
+        discrete = MAX (now - step, 0);
+        ret = backlight_helper_set_value ("set-brightness",
+                                          discrete,
+                                          error);
+        if (ret)
+                percentage_value = ABS_TO_PERCENTAGE (min, max, discrete);
+
+        return percentage_value;
+}
+
+int
+backlight_set_abs (GnomeRRScreen *rr_screen,
+                   guint value,
+                   GError **error)
+{
+        GnomeRROutput *output;
+        gboolean ret = FALSE;
+
+        /* prefer xbacklight */
+        output = get_primary_output (rr_screen);
+        if (output != NULL) {
+                ret = gnome_rr_output_set_backlight (output,
+                                                     value,
+                                                     error);
+                return ret;
+        }
+
+        /* fall back to the polkit helper */
+        ret = backlight_helper_set_value ("set-brightness",
+                                          value,
+                                          error);
+
+        return ret;
+}
+
+void
+reset_idletime (void)
+{
+        static gboolean inited = FALSE;
+        static KeyCode keycode1, keycode2;
+        static gboolean first_keycode = FALSE;
+
+        if (inited == FALSE) {
+                keycode1 = XKeysymToKeycode (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), GDK_KEY_Alt_L);
+                keycode2 = XKeysymToKeycode (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), GDK_KEY_Alt_R);
+        }
+
+        gdk_error_trap_push ();
+        /* send a left or right alt key; first press, then release */
+        XTestFakeKeyEvent (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), first_keycode ? keycode1 : keycode2, True, CurrentTime);
+        XTestFakeKeyEvent (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), first_keycode ? keycode1 : keycode2, False, CurrentTime);
+        first_keycode = !first_keycode;
+        gdk_error_trap_pop_ignored ();
+}
+
+static gboolean
+randr_output_is_on (GnomeRROutput *output)
+{
+        GnomeRRCrtc *crtc;
+
+        crtc = gnome_rr_output_get_crtc (output);
+        if (!crtc)
+                return FALSE;
+        return gnome_rr_crtc_get_current_mode (crtc) != NULL;
+}
+
+gboolean
+external_monitor_is_connected (GnomeRRScreen *screen)
+{
+        GnomeRROutput **outputs;
+        guint i;
+
+#ifdef GSD_MOCK
+	char *mock_external_monitor_contents;
+
+	if (g_file_get_contents ("GSD_MOCK_EXTERNAL_MONITOR", &mock_external_monitor_contents, NULL, NULL)) {
+		if (mock_external_monitor_contents[0] == '1') {
+			g_free (mock_external_monitor_contents);
+			return TRUE;
+		} else if (mock_external_monitor_contents[0] == '0') {
+			g_free (mock_external_monitor_contents);
+			return FALSE;
+		}
+
+		g_warning ("Unhandled value for GSD_MOCK_EXTERNAL_MONITOR contents: %s", mock_external_monitor_contents);
+		g_free (mock_external_monitor_contents);
+	}
+#endif /* GSD_MOCK */
+
+        /* see if we have more than one screen plugged in */
+        outputs = gnome_rr_screen_list_outputs (screen);
+        for (i = 0; outputs[i] != NULL; i++) {
+                if (randr_output_is_on (outputs[i]) &&
+                    !gnome_rr_output_is_laptop (outputs[i]))
+                        return TRUE;
+        }
+
+        return FALSE;
+}
+
+static void
+play_sound (void)
+{
+        ca_context_play (ca_gtk_context_get (), UPS_SOUND_LOOP_ID,
+                         CA_PROP_EVENT_ID, "battery-caution",
+                         CA_PROP_EVENT_DESCRIPTION, _("Battery is critically low"), NULL);
+}
+
+static gboolean
+play_loop_timeout_cb (gpointer user_data)
+{
+        play_sound ();
+        return TRUE;
+}
+
+void
+play_loop_start (guint *id)
+{
+        if (*id != 0)
+                return;
+
+        *id = g_timeout_add_seconds (GSD_POWER_MANAGER_CRITICAL_ALERT_TIMEOUT,
+                                     (GSourceFunc) play_loop_timeout_cb,
+                                     NULL);
+        play_sound ();
+}
+
+void
+play_loop_stop (guint *id)
+{
+        if (*id == 0)
+                return;
+
+        ca_context_cancel (ca_gtk_context_get (), UPS_SOUND_LOOP_ID);
+        g_source_remove (*id);
+        *id = 0;
 }
